from library import Library
from ..nodes import Generator
from conditional import Conditional

class TabSwitch(Library):
	dependencies = ["KeyPress"]

	def __init__(self, layout):
		super(TabSwitch, self).__init__(layout)

		self.focusable = []
		self.focused_id = 0

	def loop(self, instances):
		key = instances["KeyPress"].getKey()
		if key == 9: # Tab
			self.tab()

	def tab(self):
		if self.focused_id >= len(self.focusable):
			self.focused_id = 0

		if self.focused_id >= len(self.focusable):
			return

		old = self.focusable[self.focused_id]

		self.focused_id += 1
		if self.focused_id >= len(self.focusable):
			self.focused_id = 0

		new = self.focusable[self.focused_id]

		old.revoke()
		new.revoke()

	class Focusable(Generator):
		container = True
		text_container = False

		def init(self, **kwargs):
			self.layout.libs["TabSwitch"].focusable.append(self)

		def onGenerate(self):
			from ..xml_parser import handleElement

			if len(self.children) != 1:
				raise ValueError("<Focusable> can contain only one node")

			slots = dict(**self.slots)
			slots["focused"] = "True" if self.layout.libs["TabSwitch"].focusable.index(self) == self.layout.libs["TabSwitch"].focused_id else "False"
			return handleElement(self.children[0], self.defines, slots, additional_nodes=self.additional_nodes, global_slots=self.global_slots)


from asteval import Interpreter

def safeEval(expr, scope):
	scope = dict(**scope)
	for name in scope:
		if getattr(scope[name], "wrapped", False) is True:
			scope[name] = scope[name](scope)

	interpreter = Interpreter(
		usersyms=scope,
		use_numpy=False,
		minimal=False
	)

	res = interpreter.eval(expr, show_errors=False)

	if interpreter.error:
		raise interpreter.error

	return res


from library import Library
from ..nodes import Generator, Container

class Loops(Library):
	class Range(Generator):
		container = True
		text_container = False

		def init(self, from_, to, step=1, slot=None):
			self.from_ = from_
			self.to = to
			self.step = step
			self.slot = slot

		def onGenerate(self):
			from ..xml_parser import handleElement

			try:
				from_ = int(self.from_)
			except ValueError:
				raise ValueError("'from' attribute of <Range> must be an integer")

			try:
				to = int(self.to)
			except ValueError:
				raise ValueError("'to' attribute of <Range> must be an integer")

			try:
				step = int(self.step)
			except ValueError:
				raise ValueError("'step' attribute of <Range> must be an integer")

			to += step

			new_slots = dict(**self.slots)

			res = []
			for i in range(from_, to, step):
				if self.slot is not None:
					new_slots[self.slot] = i

				for child in self.children:
					res += handleElement(child, self.defines, dict(**new_slots), additional_nodes=self.additional_nodes, global_slots=self.global_slots)

			return res

	class For(Generator):
		container = True
		text_container = False

		def init(self, in_, slot=None):
			self.in_ = in_
			self.slot = slot

		def onGenerate(self):
			from ..xml_parser import handleElement

			try:
				in_ = list(self.in_)
			except ValueError:
				raise ValueError("'in' attribute of <For> must be a list")

			new_slots = dict(**self.slots)

			res = []
			for i in in_:
				if self.slot is not None:
					new_slots[self.slot] = i

				for child in self.children:
					res += handleElement(child, self.defines, dict(**new_slots), additional_nodes=self.additional_nodes, global_slots=self.global_slots)

			return res


from library import Library
import os

def waitKey():
	result = None
	if os.name == "nt":
		import msvcrt
		result = msvcrt.getch()
	else:
		import termios
		fd = sys.stdin.fileno()

		oldterm = termios.tcgetattr(fd)
		newattr = termios.tcgetattr(fd)
		newattr[3] = newattr[3] & ~termios.ICANON & ~termios.ECHO
		termios.tcsetattr(fd, termios.TCSANOW, newattr)

		try:
			result = sys.stdin.read(1)
		except IOError:
			pass
		finally:
			termios.tcsetattr(fd, termios.TCSAFLUSH, oldterm)

	return result

class KeyPress(Library):
	def beforeLoop(self, instances):
		self.key = None
		self.instances = instances

	def loop(self, instances):
		if self.key is None:
			self.key = ord(waitKey())
		if self.key == 3:
			raise KeyboardInterrupt

	def getKey(self):
		if self.key is None:
			self.loop(self.instances)
		return self.key